@@ -18,7 +18,7 @@
 #
 # Copyright 2001-2005 Free Software Foundation
 #
-# $Id: GCParser.py 6938 2005-02-02 14:48:02Z johannes $
+# $Id: GCParser.py 7576 2005-06-03 10:38:59Z kilo $
 
 import re
 
@@ -26,11 +26,11 @@ from gnue.common.definitions import GParser, GObjects, GRootObj
 from gnue.common.definitions.GParserHelpers import GContent
 from gnue.common.formatting import GTypecast
 from gnue.common.apps import errors
-from gnue.appserver.classrep import helpers, Namespace
+from gnue.appserver import repository
 
 
 xmlElements   = None
-_LENGTH_SCALE = re.compile ('^\w+\s*\(\s*(\d*)\s*[\,]{0,1}\s*(\d*)\s*\)\s*')
+_LENGTH_SCALE = re.compile ('^\w+\s*\(\s*(\d*)\s*[\.\,]{0,1}\s*(\d*)\s*\)\s*')
 
 # =============================================================================
 # Exceptions
@@ -140,6 +140,8 @@ def getXMLelements ():
           'type': {
             'Required': True,
             'Typecast': GTypecast.text},
+          'file': {
+            'Typecast': GTypecast.name },
           'nullable': {
             'Default' : True,
             'Typecast': GTypecast.boolean},
@@ -167,6 +169,8 @@ def getXMLelements ():
             'Typecast': GTypecast.name},
           'type': {
             'Typecast': GTypecast.text},
+          'file': {
+            'Typecast': GTypecast.name },
           'nullable': {
             'Default' : True,
             'Typecast': GTypecast.boolean},
@@ -277,11 +281,12 @@ class GCTypeDefinition (GCObject):
       fieldType = self.type.lower ().strip ()
       tpMatch   = re.compile ('^(\w+)').match (fieldType)
       if tpMatch is None:
-        raise helpers.TypeNameError, (fieldType)
+        raise repository.TypeNameError, (fieldType)
 
       typename = tpMatch.groups () [0]
-      if not typename in helpers.BASE_TYPES and len (typename.split ('_')) != 2:
-        raise helpers.TypeNameError, (fieldType)
+      if not repository.BASE_TYPES.has_key (typename) and \
+         len (typename.split ('_')) != 2:
+        raise repository.TypeNameError, (fieldType)
 
       # try to extract length and scale from fieldType
       lsMatch = _LENGTH_SCALE.match (fieldType)
@@ -298,8 +303,8 @@ class GCTypeDefinition (GCObject):
             raise DuplicateDefinitionError, (self.name, False)
           self.scale = int (sstr)
 
-      if typename in helpers.BASE_TYPES:
-        helpers.verifyBasetype (typename, self.length, self.scale)
+      if repository.BASE_TYPES.has_key (typename):
+        repository.verifyBasetype (typename, self.length, self.scale)
       else:
         if not self._refsAllowed:
           raise ReferenceNotAllowedError, (typename, self._type, self.name)
@@ -307,14 +312,14 @@ class GCTypeDefinition (GCObject):
         self.isReference = True
 
         if self.length:
-          raise helpers.TypeFormatError, \
+          raise repository.TypeFormatError, \
               u_("Reference type '%s' must not have a length") % typename
         if self.scale:
-          raise helpers.TypeFormatError, \
+          raise repository.TypeFormatError, \
               u_("Reference type '%s' must not have a scale") % typename
 
-        typename = 'string'
-        self.length = 32
+        typename    = repository.REF_TYPE
+        self.length = repository.REF_LENGTH
 
       self.datatype = typename
 
@@ -357,10 +362,12 @@ class GCClass (GCObject):
   # ---------------------------------------------------------------------------
 
   def _validate (self):
+    parent = self.getParent ()
+
     if not len (self.name):
       raise MissingNameError, u_("Class has no name")
 
-    if not isinstance (self._parent, GCModule):
+    if not isinstance (parent, GCModule):
       raise NoModuleError, self.name
 
     nameParts = self.name.split ('_')
@@ -371,14 +378,14 @@ class GCClass (GCObject):
       (self.module, self.name) = nameParts [:2]
 
     if not hasattr (self, 'module'):
-      self.module = self._parent.name
+      self.module = parent.name
 
     # if this class is not defined by the top level module, this is just a
     # definition for a class extension
-    if self.module != self._parent.name:
+    if self.module != parent.name:
       self.action = 'extend'
 
-    self.fullName = Namespace.createName (self.module, self.name)
+    self.fullName = repository.createName (self.module, self.name)
 
 
   # ---------------------------------------------------------------------------
@@ -399,7 +406,7 @@ class GCClass (GCObject):
           pType = 'string', length = 8, nullable = True)
 
       if hasattr (self, 'filter'):
-        (fModule, fName) = Namespace.splitName (self.filter)
+        (fModule, fName) = repository.splitName (self.filter)
         self.__addProperty (fName, module = fModule)
 
 
@@ -422,12 +429,17 @@ class GCClass (GCObject):
         proc._children = prop._children
 
         for child in proc._children:
-          child._parent = proc
+          child.setParent (proc)
 
         proc._validate ()
 
         self._children.remove (prop)
 
+        # Release references of the no longer needed property instance (for gc)
+        del prop._inits [:]
+        prop.setParent (None)
+
+
   # ---------------------------------------------------------------------------
   # Add a property to the class
   # ---------------------------------------------------------------------------
@@ -436,7 +448,7 @@ class GCClass (GCObject):
       length = None, nullable = False):
     items = [c.fullName for c in self.findChildrenOfType ('GCProperty')]
 
-    fullName = Namespace.createName (module, name)
+    fullName = repository.createName (module, name)
     if fullName in items:
       return None
 
@@ -455,7 +467,7 @@ class GCClass (GCObject):
 
     return prop
 
-      
+
 
 
 
@@ -477,7 +489,7 @@ class GCProperty (GCTypeDefinition):
   def _validate (self):
     GCTypeDefinition._validate (self)
     self.module   = self.findParentOfType ('GCModule').name
-    self.fullName = Namespace.createName (self.module, self.name)
+    self.fullName = repository.createName (self.module, self.name)
 
 
 # =============================================================================
@@ -498,7 +510,7 @@ class GCProcedure (GCTypeDefinition):
   def _validate (self):
     GCTypeDefinition._validate (self)
     self.module   = self.findParentOfType ('GCModule').name
-    self.fullName = Namespace.createName (self.module, self.name)
+    self.fullName = repository.createName (self.module, self.name)
 
 
 # =============================================================================
@@ -525,7 +537,7 @@ class GCIndex (GCObject):
 
   def _complete (self):
     self.module     = self.findParentOfType ('GCModule').name
-    self.fullName   = "ix_%s" % Namespace.createName (self.module, self.name)
+    self.fullName   = "ix_%s" % repository.createName (self.module, self.name)
     self.fields     = self.findChildrenOfType ('GCIndexField')
 
   def _getFieldNames (self):
@@ -542,8 +554,8 @@ class GCIndexField (GCObject):
 
   def _complete (self):
     if not '_' in self.name:
-      self.module   = self._parent.module
-      self.fullName = Namespace.createName (self.module, self.name)
+      self.module   = self.getParent ().module
+      self.fullName = repository.createName (self.module, self.name)
     else:
-      self.module   = Namespace.splitName (self.name) [0]
+      self.module   = repository.splitName (self.name) [0]
       self.fullName = self.name
