@@ -19,21 +19,21 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: readgcd.py 6998 2005-02-11 11:06:42Z johannes $
+# $Id: readgcd.py 7829 2005-08-12 14:51:18Z reinhard $
 
 import sys
 import os
-import string
 
 from gnue.common.apps import i18n, errors
+from gnue.common.apps.i18n import translate as _        # for epydoc
 from gnue.common.apps.GClientApp import *
 from gnue.common.utils.FileUtils import openResource
-from gnue.common.datasources import GDataSource
+from gnue.common.datasources import GDataSource, GSchema
 from gnue.common.utils.uuid import UUID
 
 from gnue.appserver import VERSION
 from gnue.appserver.gcd import GCParser
-from gnue.appserver.classrep import Namespace, helpers
+from gnue.appserver import repository
 from gnue.appserver import geasConfiguration
 
 
@@ -57,10 +57,10 @@ class Error (errors.ApplicationError):
     text = []
 
     if files:
-      text.append (u_("In file(s) '%s':") % string.join (files, ', '))
+      text.append (u_("In file(s) '%s':") % ', '.join (files))
 
     text.append (message)
-    self.detail = string.join (text, os.linesep)
+    self.detail = os.linesep.join (text)
 
 
 class ModuleNotFoundError (Error):
@@ -99,7 +99,7 @@ class DuplicateClassError (Error):
 
 
 # =============================================================================
-# Update a database schema from GNUe Class Definitions and update classrep.
+# Update a database schema from GNUe Class Definitions and update repository.
 # =============================================================================
 
 class gcdClient (GClientApp):
@@ -224,28 +224,28 @@ class gcdReader:
     self._connections = connections
     self._database    = database
     self._filenames   = files
-    self._files       = []
+    self._files       = {}
     self._dump        = dump
     self._fileonly    = fonly
     self._uuidType    = gConfig ('uuidtype', section = 'appserver').lower ()
 
     # ResultSets
-    self.moduleRS = None
-    self.classRS  = None
-    self.propRS   = None
-    self.procRS   = None
-    self.parmRS   = None
+    self.__moduleRS = None
+    self.__classRS  = None
+    self.__propRS   = None
+    self.__procRS   = None
 
     # Lookup dictionaries
-    self.__lpModule   = {}        # gnue_id -> name, name -> gnue_id
-    self.__lpClass    = {}        # fqName -> gnue_id
+    self.lpModule     = {}        # gnue_id -> name, name -> gnue_id
+    self.lpClass      = {}        # fqName -> gnue_id
+    self.lpProperty   = {}        # class-id.module-id_name -> gnue_id
+    self.lpProcedure  = {}        # class-name.module-name_name -> gnue_id
     self.__lpFilter   = {}        # fqName -> gnue_id
-    self.__lpProperty = {}        # class-id.module-id_name -> gnue_id
-    self.__lpProcedure= {}        # class-name.module-name_name -> gnue_id
     self.__duplos     = {}        # fqName
+    self.__tables     = {}        # tablename -> GSTable instance
 
     # Data from gcd files
-    self.schema     = {}
+    self.schema     = GSchema.GSSchema ()
     self.modules    = {}
     self.classes    = {}
     self.properties = {}
@@ -264,10 +264,11 @@ class gcdReader:
 
     try:
       for filename in files:
-        self._files.append (openResource (filename))
+        self._files [filename] = openResource (filename)
 
     except IOError, err:
-      for item in self._files:
+      while self._files:
+        (name, item) = self._files.popitem ()
         item.close ()
 
       raise StartupError, u_("Unable to open input file: %s") % \
@@ -292,15 +293,23 @@ class gcdReader:
 
     print o (u_("Loading gcd files ..."))
 
-    for item in xrange (len (self._files)):
-      try:
-        self.__currentFile = self._filenames [item]
+    while self._files:
+      self.__currentFile, stream = self._files.popitem ()
+      self.__currentClass = None
 
-        schema = GCParser.loadFile (self._files [item])
-        schema.walk (self.__iterateObjects)
+      try:
+        schema = GCParser.loadFile (stream)
+
+        try:
+          schema.walk (self.__iterateObjects)
+
+        finally:
+          self.__currentClass = None
+          schema = None
 
       finally:
-        self._files [item].close ()
+        stream.close ()
+
 
     self.__integrityCheck ()
 
@@ -324,14 +333,13 @@ class gcdReader:
     self._connections.loginToConnection (connection)
 
     print _("Updating schema ...")
-    code = connection.updateSchema (self.schema.values (), self._fileonly)
+    code = connection.writeSchema (self.schema, self._fileonly)
 
     if self._dump is not None:
       dest = open (self._dump, 'w')
 
-      for item in code:
-        for line in item:
-          dest.write (line + os.linesep)
+      for line in code:
+        dest.write (line + os.linesep)
 
       dest.close ()
 
@@ -370,7 +378,6 @@ class gcdReader:
     """
 
     if sObject._type == 'GCModule':
-      self.__currentModule = sObject
       self.__addModuleToRepository (sObject)
 
     elif sObject._type == 'GCClass':
@@ -403,21 +410,21 @@ class gcdReader:
     to ids and vice versa
     """
 
-    self.__lpModule = {}
+    self.lpModule.clear ()
 
-    if self.moduleRS is None:
-      self.moduleRS = self.__openSource ('gnue_module',
+    if self.__moduleRS is None:
+      self.__moduleRS = self.__openSource ('gnue_module',
                    ['gnue_id', 'gnue_name', 'gnue_comment']).createResultSet ()
 
-    rec = self.moduleRS.firstRecord ()
+    rec = self.__moduleRS.firstRecord ()
     while rec is not None:
       gid  = rec.getField ('gnue_id')
       name = rec.getField ('gnue_name')
 
-      self.__lpModule [gid]  = name
-      self.__lpModule [name.lower ()] = gid
+      self.lpModule [gid]  = name
+      self.lpModule [name.lower ()] = gid
 
-      rec  = self.moduleRS.nextRecord ()
+      rec  = self.__moduleRS.nextRecord ()
 
 
 
@@ -432,29 +439,29 @@ class gcdReader:
     defined it will be set into the filter dictionary.
     """
 
-    self.__lpClass  = {}
-    self.__lpFilter = {}
+    self.lpClass.clear ()
+    self.__lpFilter.clear ()
 
-    if self.classRS is None:
-      self.classRS = self.__openSource ('gnue_class', ['gnue_id', 'gnue_name',
+    if self.__classRS is None:
+      self.__classRS = self.__openSource ('gnue_class', ['gnue_id', 'gnue_name',
           'gnue_module', 'gnue_comment', 'gnue_filter']).createResultSet ()
 
-    rec = self.classRS.firstRecord ()
+    rec = self.__classRS.firstRecord ()
     while rec is not None:
       gid  = rec.getField ('gnue_id')
       name = rec.getField ('gnue_name')
       mid  = rec.getField ('gnue_module')
       fid  = rec.getField ('gnue_filter')
 
-      fqName = Namespace.createName (self.__lpModule [mid], name)
+      fqName = repository.createName (self.lpModule [mid], name)
 
-      self.__lpClass [gid]    = fqName
-      self.__lpClass [fqName.lower ()] = gid
+      self.lpClass [gid]    = fqName
+      self.lpClass [fqName.lower ()] = gid
 
       if fid is not None:
         self.__lpFilter [fqName.lower ()] = fid
 
-      rec  = self.classRS.nextRecord ()
+      rec  = self.__classRS.nextRecord ()
 
 
 
@@ -468,23 +475,23 @@ class gcdReader:
     constructed from "classid.moduleid_propertyname".
     """
 
-    self.__lpProperty = {}
+    self.lpProperty.clear ()
 
-    if self.propRS is None:
-      self.propRS = self.__openSource ('gnue_property', ['gnue_id',
+    if self.__propRS is None:
+      self.__propRS = self.__openSource ('gnue_property', ['gnue_id',
           'gnue_module', 'gnue_class', 'gnue_name', 'gnue_type', 'gnue_length',
           'gnue_scale', 'gnue_nullable', 'gnue_comment']).createResultSet ()
 
-    rec = self.propRS.firstRecord ()
+    rec = self.__propRS.firstRecord ()
     while rec is not None:
       mid  = rec.getField ('gnue_module')
       cid  = rec.getField ('gnue_class')
       name = rec.getField ('gnue_name')
       key  = "%s.%s_%s" % (cid, mid, name.lower ())
 
-      self.__lpProperty [key] = rec.getField ('gnue_id')
+      self.lpProperty [key] = rec.getField ('gnue_id')
 
-      rec  = self.propRS.nextRecord ()
+      rec  = self.__propRS.nextRecord ()
 
 
   # ---------------------------------------------------------------------------
@@ -497,24 +504,24 @@ class gcdReader:
     from "classname.modulename_procedurename".
     """
 
-    self.__lpProcedure = {}
+    self.lpProcedure.clear ()
 
-    if self.procRS is None:
-      self.procRS = self.__openSource ('gnue_procedure', ['gnue_id',
+    if self.__procRS is None:
+      self.__procRS = self.__openSource ('gnue_procedure', ['gnue_id',
         'gnue_module', 'gnue_class', 'gnue_name', 'gnue_type', 'gnue_length',
         'gnue_scale', 'gnue_nullable', 'gnue_comment', 'gnue_code',
         'gnue_language']).createResultSet ()
 
-    rec = self.procRS.firstRecord ()
+    rec = self.__procRS.firstRecord ()
     while rec is not None:
-      mName = self.__lpModule.get (rec.getField ('gnue_module'))
-      cName = self.__lpClass.get (rec.getField ('gnue_class'))
+      mName = self.lpModule.get (rec.getField ('gnue_module'))
+      cName = self.lpClass.get (rec.getField ('gnue_class'))
       pName = rec.getField ('gnue_name')
       key   = "%s.%s_%s" % (cName, mName, pName)
 
-      self.__lpProcedure [key.lower ()] = rec.getField ('gnue_id')
+      self.lpProcedure [key.lower ()] = rec.getField ('gnue_id')
 
-      rec  = self.procRS.nextRecord ()
+      rec  = self.__procRS.nextRecord ()
 
 
   # ---------------------------------------------------------------------------
@@ -531,7 +538,7 @@ class gcdReader:
 
     mkey = item.name.lower ()
     mDef = self.__fetchTags (item, ['name', 'comment'])
-    mDef ['gnue_id'] = self.__lpModule.get (mkey)
+    mDef ['gnue_id'] = self.lpModule.get (mkey)
 
     if not self.modules.has_key (mkey):
       self.modules [mkey] = {}
@@ -553,15 +560,16 @@ class gcdReader:
     """
 
     ckey = item.fullName.lower ()
-    if not self.schema.has_key (ckey):
-      self.schema [ckey] = {'name': item.fullName, 'fields': []}
+    if not ckey in self.__tables:
+      parent = self.schema.findChildOfType ('GSTables') or \
+          GSchema.GSTables (self.schema, type = "table")
+      self.__tables [ckey] = GSchema.GSTable (parent, name = item.fullName)
+      pk = GSchema.GSPrimaryKey (self.__tables [ckey],
+                                                name = "pk_%s" % item.fullName)
+      GSchema.GSPKField (pk, name = u"gnue_id")
 
-    if item.action == 'create':
-      self.schema [ckey] ['primarykey'] = { \
-          'name'  : 'pk_%s' % item.fullName,
-          'fields': ['gnue_id']}
-
-      if self.__duplos.has_key (ckey):
+    if item.action == "create":
+      if ckey in self.__duplos:
         raise DuplicateClassError, \
             (item.fullName, self.__currentFile, self.__duplos [ckey])
 
@@ -579,13 +587,12 @@ class gcdReader:
     @param item: GCIndex instance with the index to be added
     """
 
-    classDef = self.schema [self.__currentClass.fullName.lower ()]
-    if not classDef.has_key ('indices'):
-      classDef ['indices'] = []
+    table  = self.__tables [self.__currentClass.fullName.lower ()]
+    parent = table.findChildOfType ('GSIndexes') or GSchema.GSIndexes (table)
 
-    classDef ['indices'].append ({'name'  : item.fullName,
-                                  'unique': item.unique,
-                                  'fields': item.fieldNames})
+    ix = GSchema.GSIndex (parent, name = item.fullName, unique = item.unique)
+    for field in item.fieldNames:
+      GSchema.GSIndexField (ix, name = field)
 
 
   # ---------------------------------------------------------------------------
@@ -602,7 +609,7 @@ class gcdReader:
 
     key  = item.fullName.lower ()
     cDef = self.__fetchTags (item, ['module', 'name', 'comment', 'filter'])
-    cDef ['gnue_id'] = self.__lpClass.get (key)
+    cDef ['gnue_id'] = self.lpClass.get (key)
 
     if not self.classes.has_key (key):
       self.classes [key] = {'_files': []}
@@ -623,7 +630,9 @@ class gcdReader:
     @param item: GCProperty instance to add
     """
 
-    classDef = self.schema [item._parent.fullName.lower ()]
+    parent = item.getParent ()
+    table  = self.__tables [parent.fullName.lower ()]
+    master = table.findChildOfType ('GSFields') or GSchema.GSFields (table)
 
     pDef = {'name'    : item.fullName,
             'type'    : item.datatype,
@@ -632,19 +641,18 @@ class gcdReader:
     if item.length: pDef ['length']    = item.length
     if item.scale:  pDef ['precision'] = item.scale
 
-    classDef ['fields'].append (pDef)
+    GSchema.GSField (master, **pDef)
 
     # If the property is a reference to another class, add a constraint to the
     # class definition
     if item.isReference:
-      if not classDef.has_key ('constraints'):
-        classDef ['constraints'] = []
+      master = table.findChildOfType ('GSConstraints') or \
+                                                  GSchema.GSConstraints (table)
+      fkey = GSchema.GSForeignKey (master,
+          name = "fk_%s_%s" % (parent.fullName, item.fullName),
+          references = item.type)
 
-      classDef ['constraints'].append ( \
-          {'name'     : "fk_%s_%s" % (item._parent.fullName, item.fullName),
-           'fields'   : [item.fullName],
-           'reftable' : item.type,
-           'reffields': ['gnue_id']})
+      GSchema.GSFKField (fkey, name = item.fullName, references = u"gnue_id")
 
 
   # ---------------------------------------------------------------------------
@@ -659,12 +667,13 @@ class gcdReader:
     @param item: GCProperty instance to add
     """
 
-    fqName = "%s.%s" % (item._parent.fullName, item.fullName)
-    pkey   = "%s.%s_%s" % (self.__lpClass.get (item._parent.fullName.lower ()),
-                         self.__lpModule.get (item.module.lower ()), item.name)
+    parent = item.getParent ()
+    fqName = "%s.%s" % (parent.fullName, item.fullName)
+    pkey   = "%s.%s_%s" % (self.lpClass.get (parent.fullName.lower ()),
+                         self.lpModule.get (item.module.lower ()), item.name)
 
-    pDef = {'gnue_id'   : self.__lpProperty.get (pkey.lower ()),
-            'gnue_class': item._parent.fullName,
+    pDef = {'gnue_id'   : self.lpProperty.get (pkey.lower ()),
+            'gnue_class': parent.fullName,
             'gnue_type' : item.datatype}
 
     pDef.update (self.__fetchTags (item,
@@ -691,9 +700,10 @@ class gcdReader:
     @param item: GCProcedure instance to add
     """
 
-    fqName = "%s.%s" % (item._parent.fullName, item.fullName)
+    parent = item.getParent ()
+    fqName = "%s.%s" % (parent.fullName, item.fullName)
     pDef = {'gnue_id'   : None,
-            'gnue_class': item._parent.fullName,
+            'gnue_class': parent.fullName,
             'gnue_type' : item.datatype,
             'gnue_code' : item.getChildrenAsContent ()}
 
@@ -715,7 +725,8 @@ class gcdReader:
     @param item: GCParameter instance to add
     """
 
-    pName  = "%s.%s" % (self.__currentClass.fullName, item._parent.fullName)
+    parent = item.getParent ()
+    pName  = "%s.%s" % (self.__currentClass.fullName, parent.fullName)
     fqName = "%s.%s" % (pName, item.name)
 
     par = {'gnue_procedure': pName, 'gnue_type': item.datatype}
@@ -738,20 +749,20 @@ class gcdReader:
       # Make sure the module referenced is a valid module, either an existing
       # or a new one
       mname = item ['gnue_module'].lower ()
-      if not self.__lpModule.get (mname) and not self.modules.get (mname):
+      if not self.lpModule.get (mname) and not self.modules.get (mname):
         raise ModuleNotFoundError, (item ['gnue_module'], item ['_files'])
 
       # Make sure the filter class will be available if one is defined
       if item.get ('gnue_filter') is not None:
-        fname = item ['gnue_filter']
-        if not self.__lpClass.get (fname) and not self.classes.get (fname):
+        fname = item ['gnue_filter'].lower ()
+        if not self.lpClass.get (fname) and not self.classes.get (fname):
           raise FilterNotFoundError, (name, fname, item ['_files'])
 
       # For an existing class we need to make sure the filter-attribute won't
       # get changed, because we cannot add a not NULL field to an existing class
       if item ['gnue_id'] is not None:
         iFilter = item.get ('gnue_filter') and \
-                  self.__lpClass [item ['gnue_filter'].lower ()]
+                  self.lpClass [item ['gnue_filter'].lower ()]
 
         if iFilter != self.__lpFilter.get (name):
           raise FilterChangeError, (name, item ['_files'])
@@ -762,9 +773,9 @@ class gcdReader:
       classDef = self.classes [item ['gnue_class'].lower ()]
 
       # If a property is a reference, make sure the referenced class exists
-      if not item ['gnue_type'] in helpers.BASE_TYPES:
+      if not repository.BASE_TYPES.has_key (item ['gnue_type']):
         refname = item ['gnue_type'].lower ()
-        if not self.__lpClass.get (refname) and not self.classes.get (refname):
+        if not self.lpClass.get (refname) and not self.classes.get (refname):
           raise ClassNotFoundError, (item ['gnue_type'], classDef ['_files'])
 
       # For an existing class make sure to *not* add NOT NULL properties 
@@ -793,14 +804,19 @@ class gcdReader:
     """
 
     # Update the result set and the lookup dictionary
-    stat = self.__processResultSet (self.moduleRS, self.modules, ['gnue_name'])
-    self.__getModuleLookup ()
+    try:
+      stat = self.__processResultSet (self.__moduleRS, self.modules,
+                                      ['gnue_name'])
+      self.__getModuleLookup ()
 
-    print o (u_("  Modules   : %(ins)3d inserted, %(upd)3d updated, %(kept)3d "
-                "unchanged.") \
-             % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]})
+      print o (u_("  Modules   : %(ins)3d inserted, %(upd)3d updated, "
+                  "%(kept)3d unchanged.") \
+               % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]})
 
-    return (stat [0] + stat [1]) > 0
+      return (stat [0] + stat [1]) > 0
+
+    finally:
+      self.__moduleRS.close ()
   
 
   # ---------------------------------------------------------------------------
@@ -815,49 +831,53 @@ class gcdReader:
     @return: True if a commit is needed, False otherwise
     """
 
-    # First make sure we have valid 'gnue_module' entries for all classes
-    self.__replaceReferences (self.classes, self.__lpModule, 'gnue_module')
+    try:
+      # First make sure we have valid 'gnue_module' entries for all classes
+      self.__replaceReferences (self.classes, self.lpModule, 'gnue_module')
 
-    # Then have a look at the filters defined on the classes
-    filterStack = []
+      # Then have a look at the filters defined on the classes
+      filterStack = []
 
-    for (fullName, item) in self.classes.items ():
-      if item.has_key ('gnue_filter'):
-        filterName = item ['gnue_filter']
-        filterId   = self.__lpClass.get (filterName)
+      for (fullName, item) in self.classes.items ():
+        if item.has_key ('gnue_filter'):
+          filterName = item ['gnue_filter'].lower ()
+          filterId   = self.lpClass.get (filterName)
 
-        # If the filter-class is available we can use it
-        if filterId is not None:
-          item ['gnue_filter'] = filterId
-        else:
-          # since the filter-id will be created later, we add it to the queue
-          # and remove the attribute from the class.
-          filterStack.append ((item, filterName, fullName))
-          del item ['gnue_filter']
+          # If the filter-class is available we can use it
+          if filterId is not None:
+            item ['gnue_filter'] = filterId
+          else:
+            # since the filter-id will be created later, we add it to the queue
+            # and remove the attribute from the class.
+            filterStack.append ((item, filterName, fullName))
+            del item ['gnue_filter']
 
-    # Merge all changes into the current result set
-    cond = ['gnue_module', 'gnue_name']
-    stat = self.__processResultSet (self.classRS, self.classes, cond)
-    needCommit = (stat [0] + stat [1]) > 0
+      # Merge all changes into the current result set
+      cond = ['gnue_module', 'gnue_name']
+      stat = self.__processResultSet (self.__classRS, self.classes, cond)
+      needCommit = (stat [0] + stat [1]) > 0
 
-    # Update the lookup dictionary for classes
-    self.__getClassLookup ()
+      # Update the lookup dictionary for classes
+      self.__getClassLookup ()
 
-    # Process all classes with a 'new' filter class
-    filterChanged = False
+      # Process all classes with a 'new' filter class
+      filterChanged = False
 
-    if filterStack:
-      for (item, filterName, fullName) in filterStack:
-        item ['gnue_filter'] = self.__lpClass.get (filterName)
+      if filterStack:
+        for (item, filterName, fullName) in filterStack:
+          item ['gnue_filter'] = self.lpClass.get (filterName)
 
-      fst = self.__processResultSet (self.classRS, self.classes, cond)
-      filterChanged = fst [1] > 0
+        fst = self.__processResultSet (self.__classRS, self.classes, cond)
+        filterChanged = fst [1] > 0
 
-    print o (u_("  Classes   : %(ins)3d inserted, %(upd)3d updated, %(kept)3d "
-                "unchanged.") \
-          % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]})
+      print o (u_("  Classes   : %(ins)3d inserted, %(upd)3d updated, "
+                  "%(kept)3d unchanged.") \
+            % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]})
 
-    return needCommit or filterChanged
+      return needCommit or filterChanged
+
+    finally:
+      self.__classRS.close ()
 
 
   # ---------------------------------------------------------------------------
@@ -871,19 +891,24 @@ class gcdReader:
     @return: True if a commit is needed, False otherwise
     """
 
-    # Make sure all properties have a valid 'gnue_module' and 'gnue_class'
-    self.__replaceReferences (self.properties, self.__lpModule, 'gnue_module')
-    self.__replaceReferences (self.properties, self.__lpClass, 'gnue_class')
+    try:
+      # Make sure all properties have a valid 'gnue_module' and 'gnue_class'
+      self.__replaceReferences (self.properties, self.lpModule, 'gnue_module')
+      self.__replaceReferences (self.properties, self.lpClass, 'gnue_class')
 
-    # Load and update all properties
-    cond = ['gnue_module', 'gnue_class', 'gnue_name']
-    stat = self.__processResultSet (self.propRS, self.properties, cond)
+      # Load and update all properties
+      cond = ['gnue_module', 'gnue_class', 'gnue_name']
+      stat = self.__processResultSet (self.__propRS, self.properties, cond)
+      self.__getPropertyLookup ()
 
-    print o (u_("  Properties: %(ins)3d inserted, %(upd)3d updated, %(kept)3d "
-                "unchanged.") \
-             % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]})
+      print o (u_("  Properties: %(ins)3d inserted, %(upd)3d updated, "
+                  "%(kept)3d unchanged.") \
+               % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]})
 
-    return (stat [0] + stat [1]) > 0
+      return (stat [0] + stat [1]) > 0
+
+    finally:
+      self.__propRS.close ()
 
 
   # ---------------------------------------------------------------------------
@@ -898,20 +923,24 @@ class gcdReader:
     @return: True if a commit is needed, False otherwise
     """
 
-    # Make sure all procedures have a valid 'gnue_module' and 'gnue_class'
-    self.__replaceReferences (self.procedures, self.__lpModule, 'gnue_module')
-    self.__replaceReferences (self.procedures, self.__lpClass, 'gnue_class')
+    try:
+      # Make sure all procedures have a valid 'gnue_module' and 'gnue_class'
+      self.__replaceReferences (self.procedures, self.lpModule, 'gnue_module')
+      self.__replaceReferences (self.procedures, self.lpClass, 'gnue_class')
 
-    # Load and update all procedures
-    cond = ['gnue_module', 'gnue_class', 'gnue_name']
-    stat = self.__processResultSet (self.procRS, self.procedures, cond)
-    self.__getProcedureLookup ()
+      # Load and update all procedures
+      cond = ['gnue_module', 'gnue_class', 'gnue_name']
+      stat = self.__processResultSet (self.__procRS, self.procedures, cond)
+      self.__getProcedureLookup ()
 
-    print o (u_("  Procedures: %(ins)3d inserted, %(upd)3d updated, %(kept)3d "
-                "unchanged.") \
-              % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]})
+      print o (u_("  Procedures: %(ins)3d inserted, %(upd)3d updated, "
+                  "%(kept)3d unchanged.") \
+                % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]})
 
-    return (stat [0] + stat [1]) > 0
+      return (stat [0] + stat [1]) > 0
+
+    finally:
+      self.__procRS.close ()
 
 
   # ---------------------------------------------------------------------------
@@ -926,22 +955,26 @@ class gcdReader:
     """
 
     # Make sure all parameters have a valid 'gnue_procedure'
-    self.__replaceReferences (self.parameters, self.__lpProcedure,
+    self.__replaceReferences (self.parameters, self.lpProcedure,
         'gnue_procedure')
 
-    self.parmRS = self.__openSource ('gnue_parameter', ['gnue_id',
+    resultSet = self.__openSource ('gnue_parameter', ['gnue_id',
         'gnue_procedure', 'gnue_name', 'gnue_type', 'gnue_scale',
         'gnue_length', 'gnue_comment']).createResultSet ()
 
-    # Load and update all parameters
-    cond = ['gnue_procedure', 'gnue_name']
-    stat = self.__processResultSet (self.parmRS, self.parameters, cond)
+    try:
+      # Load and update all parameters
+      cond = ['gnue_procedure', 'gnue_name']
+      stat = self.__processResultSet (resultSet, self.parameters, cond)
 
-    print o (u_("  Parameters: %(ins)3d inserted, %(upd)3d updated, %(kept)3d "
-                "unchanged.") \
+      print o (u_("  Parameters: %(ins)3d inserted, %(upd)3d updated, "
+                  "%(kept)3d unchanged.") \
               % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]})
 
-    return (stat [0] + stat [1]) > 0
+      return (stat [0] + stat [1]) > 0
+
+    finally:
+      resultSet.close ()
 
 
   # ---------------------------------------------------------------------------
@@ -987,8 +1020,7 @@ class gcdReader:
                                           {'name'      : "dts_%s" % classname,
                                            'database'  : self._database,
                                            'table'     : classname,
-                                           'primarykey': 'gnue_id'},
-                                          unicodeMode = True)
+                                           'primarykey': 'gnue_id'})
 
 
   # ---------------------------------------------------------------------------
@@ -1062,6 +1094,7 @@ class gcdReader:
 
     if needPost:
       resultSet.post ()
+      resultSet.requery (False)
 
     return stat
 
@@ -1135,6 +1168,5 @@ class gcdReader:
       return UUID.generateRandom ()
 
 
-
 if __name__ == "__main__":
   gcdClient ().run ()
