@@ -19,9 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: geasAuthentication.py 6851 2005-01-03 20:59:28Z jcater $
-
-import string
+# $Id: geasAuthentication.py 7641 2005-06-22 13:58:32Z johannes $
 
 from gnue.common.apps import errors
 
@@ -110,7 +108,7 @@ class geasDBAuthAgent (geasAuthAgent):
     gDebug (1, "User '%s' logged in." % user)
 
     # possibly not the best solution
-    session.tablelist = string.split (authData[0][3],',')
+    session.tablelist = authData[0][3].split (',')
 
     return 1 # = has access
 
