@@ -19,21 +19,19 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: geasGsdGen.py 6995 2005-02-11 10:46:57Z johannes $
+# $Id: geasGsdGen.py 7641 2005-06-22 13:58:32Z johannes $
 
 import sys
-import types
 import copy
-import string
 import mx.DateTime
 
 if sys.version_info [:2] >= (2, 3):
   import datetime
 
 from gnue.common.apps import i18n, errors
-from gnue.common.schema import Objects
+from gnue.common.apps.i18n import translate as _        # for epydoc
 from gnue.common.apps.GClientApp import *
-from gnue.common.datasources import GDataSource, GConditions
+from gnue.common.datasources import GDataSource, GConditions, GSchema
 from gnue.common.definitions import GParserHelpers
 
 from gnue.appserver import VERSION
@@ -156,11 +154,11 @@ class geasGsdGen (GClientApp):
 
     print o(u_("Generating schema definition ..."))
 
-    schema = Objects.GSSchema ()
+    schema = GSchema.GSSchema ()
     schema.title   = 'Appserver Data Dump'
     schema.author  = self.COMMAND
     schema.version = '1.0'
-    self._data = Objects.GSData (schema)
+    self._data = GSchema.GSData (schema)
 
     for classname in self.exports:
       self.__exportClass (classname)
@@ -184,30 +182,24 @@ class geasGsdGen (GClientApp):
     @param className: fully qualified name of the class to be exported
     """
 
-    cDef  = self.sm.classes [className]
+    cDef  = self.sm.repository.classes [className]
     print o(u_("Exporting data of class '%s' ...") % className)
 
     # Prepare the tabledata- and it's definition tags
-    table = Objects.GSTableData (self._data)
+    table = GSchema.GSTableData (self._data)
     table.name      = "%s_dump" % cDef.table
     table.tablename = cDef.table
 
-    columns = Objects.GSDefinition (table)
+    pk = GSchema.GSPrimaryKey (table, name = u"pk_%s" % cDef.table)
+    GSchema.GSPKField (pk, name = u'gnue_id')
 
     fieldlist = []
     for prop in cDef.properties.values ():
       if prop.isCalculated: continue
 
-      column = Objects.GSColumn (columns)
-      column.field = prop.column
-      column.type  = prop.dbFullType
-
-      if prop.fullName == 'gnue_id':
-        column.key = True
-
       fieldlist.append (prop.column)
 
-    rows = Objects.GSRows (table)
+    rows = GSchema.GSRows (table)
 
     if self.__fishes.has_key (className):
       self.__fishDataDump (cDef, fieldlist, rows)
@@ -255,7 +247,7 @@ class geasGsdGen (GClientApp):
     @param rows: GSRows instance which is parent of the newly created rows
     """
 
-    row = Objects.GSRow (rows)
+    row = GSchema.GSRow (rows)
 
     for field in fields:
       pName = "." in field and field.split (".", 1) [-1] or field
@@ -263,7 +255,7 @@ class geasGsdGen (GClientApp):
       data  = record.getField (field)
 
       if data is not None:
-        value = Objects.GSValue (row)
+        value = GSchema.GSValue (row)
         value.field = prop.column
 
         GParserHelpers.GContent ( \
@@ -342,12 +334,12 @@ class geasGsdGen (GClientApp):
     """
 
     if datatype [:6] == "string" or datatype == "id":
-      checktype (native, [types.NoneType, types.UnicodeType])
+      checktype (native, [None, unicode])
 
       if native is None:
         return u''
 
-      if isinstance (native, types.UnicodeType):
+      if isinstance (native, unicode):
         return native
 
     elif datatype [:6] == "number":
@@ -470,12 +462,13 @@ class geasGsdGen (GClientApp):
     result = {}
     fNames = {}
 
+    repository = self.sm.repository
     # first replace all class-names by there gnue-id's
     for (filterName, data) in self.__filterParams.items ():
-      if not self.sm.classes.has_key (filterName):
+      if not repository.classes.has_key (filterName):
         raise StartupError, u_("Filter class '%s' not found") % filterName
 
-      fc = self.sm.classes [filterName]
+      fc = repository.classes [filterName]
       for field in data.keys ():
         if not fc.properties.has_key (field):
           raise StartupError, \
@@ -538,23 +531,25 @@ class geasGsdGen (GClientApp):
     @param args: list of classnames or modulenames to be exported
     """
 
+    repository = self.sm.repository
+
     if not len (args):
       if self.__useFilter:
         for filterId in self.filters.keys ():
-          filterClass = self.sm.classes.find (filterId)
+          filterClass = repository.classes [filterId]
           if not filterClass.fullName in args:
             args.append (filterClass.fullName)
 
       else:
-        for c in self.sm.classes.values ():
-          if c.module.fullName == 'gnue':
+        for c in repository.classes.values ():
+          if c.gnue_module.fullName == 'gnue':
             continue
           
           args.append (c.fullName)
 
     if self.OPTIONS ['system']:
-      for c in self.sm.classes.values ():
-        if c.module.fullName == 'gnue' and not c.fullName in args:
+      for c in repository.classes.values ():
+        if c.gnue_module.fullName == 'gnue' and not c.fullName in args:
           args.append (c.fullName)
 
     self.__classes = {}
@@ -585,7 +580,8 @@ class geasGsdGen (GClientApp):
     dependencies given by class-references.
     """
 
-    cDef = self.sm.classes [className]
+    repository = self.sm.repository
+    cDef       = repository.classes [className]
 
     # every class has a sequence of master-classes
     if not self.__classes.has_key (className):
@@ -612,7 +608,7 @@ class geasGsdGen (GClientApp):
 
     # if 'include-details' is set, we've to add all detail-classes of the
     # current class.
-    for cClass in self.sm.classes.values ():
+    for cClass in repository.classes.values ():
       for p in cClass.properties.values ():
         if p.isReference and p.referencedClass.gnue_id == cDef.gnue_id:
           if not self.__classes.has_key (cClass.fullName):
@@ -675,8 +671,10 @@ class geasGsdGen (GClientApp):
 
     If destination-relation is None, destination-item is a field-value
     otherwise it's a field-name. 
-    Examples: (foo, bar, foobar, barbaz) means foo.bar = foobar.barbaz
-              (foo, bar, None, barbaz)   means foo.bar = barbaz
+
+    Examples:
+      - (foo, bar, foobar, barbaz) means foo.bar = foobar.barbaz
+      - (foo, bar, None, barbaz)   means foo.bar = barbaz
 
     @return: dictionary with conditions
     """
@@ -687,7 +685,7 @@ class geasGsdGen (GClientApp):
       return result
 
     for className in self.exports:
-      classdef = self.sm.classes [className]
+      classdef = self.sm.repository.classes [className]
 
       cond = []
 
@@ -744,27 +742,28 @@ class geasGsdGen (GClientApp):
     @return: tuple with the datasource and it's field list
     """
 
-    className = classdef.fullName
-    alias     = {classdef.table: ""}
+    className  = classdef.fullName
+    alias      = {classdef.table: ""}
+    repository = self.sm.repository
 
     # prepare a dictionary with aliases for all tables needed
     if self.conditions.has_key (className):
       alias [classdef.table] = 't0'
       index = 1
       for (srcrel, srcitem, dstrel, dstitem) in self.conditions [className]:
-        if not alias.has_key (self.sm.classes [srcrel].table):
-          alias [self.sm.classes [srcrel].table] = "t%d" % index
+        if not alias.has_key (repository.classes [srcrel].table):
+          alias [repository.classes [srcrel].table] = "t%d" % index
           index += 1
 
         if dstrel is not None and \
-            not alias.has_key (self.sm.classes [dstrel].table):
-          alias [self.sm.classes [dstrel].table] = "t%d" % index
+            not alias.has_key (repository.classes [dstrel].table):
+          alias [repository.classes [dstrel].table] = "t%d" % index
           index += 1
 
     # make sure the list of tables is sorted by alias
     tl = [(al, name) for (name, al) in alias.items ()]
     tl.sort ()
-    table = string.join (["%s %s" % (t, a.strip ()) for (a, t) in tl], ", ")
+    table = ", ".join (["%s %s" % (t, a.strip ()) for (a, t) in tl])
 
     # add alias information to the field list
     if len (alias [classdef.table]):
@@ -774,16 +773,16 @@ class geasGsdGen (GClientApp):
     conditions = None
     if self.conditions.has_key (className):
       for (srcrel, srcitem, dstrel, dstitem) in self.conditions [className]:
-        sValue = "%s.%s" % (alias [self.sm.classes [srcrel].table], srcitem)
+        sValue = "%s.%s" % (alias [repository.classes [srcrel].table], srcitem)
         if dstrel is None:
           dType  = 'const'
           dValue = dstitem
         else:
           dType  = 'field'
-          dValue = "%s.%s" % (alias [self.sm.classes [dstrel].table], dstitem)
+          dValue = "%s.%s" % (alias[repository.classes [dstrel].table], dstitem)
 
-        c = GConditions.buildTreeFromList ( \
-            ['eq', ['field', sValue], [dType, dValue]])
+        c = GConditions.buildConditionFromPrefix ( \
+                                    ['eq', ['field', sValue], [dType, dValue]])
 
         if conditions is not None:
           conditions = GConditions.combineConditions (conditions, c)
@@ -795,7 +794,7 @@ class geasGsdGen (GClientApp):
                       'database'  : self.sm._internalSession.database,
                       'table'     : table,
                       'primarykey': 'gnue_id'},
-        fields = fieldList, unicodeMode = True)
+        fields = fieldList)
     
     if conditions is not None:
       result.setCondition (conditions)
