@@ -19,10 +19,8 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: geasInstance.py 7101 2005-03-07 07:14:11Z reinhard $
+# $Id: geasInstance.py 7795 2005-08-07 18:44:59Z reinhard $
 
-import types
-import string
 import sys
 import mx.DateTime
 import mx.DateTime.ISO
@@ -33,7 +31,6 @@ if sys.version_info [:2] >= (2, 3):
 from gnue import appserver
 from gnue.appserver.language import Object, Session
 from gnue.common.logic.language import getLanguageAdapter
-from gnue.appserver.classrep import Namespace
 from gnue.common.apps import errors
 
 # =============================================================================
@@ -104,6 +101,9 @@ class geasInstance:
     self.__record     = record
     self.__classdef   = classdef
 
+    self.__connection.setConstraints (classdef.table, classdef.masters)
+
+
   # ---------------------------------------------------------------------------
   # Convert a value into the given type if possible
   # ---------------------------------------------------------------------------
@@ -116,9 +116,9 @@ class geasInstance:
 
     elif propertydef.dbType == "string":
       # String property: we expect the database to deliver a unicode string
-      if isinstance (value, types.UnicodeType):
+      if isinstance (value, unicode):
         return value
-      elif isinstance (value, types.StringType):
+      elif isinstance (value, str):
         return unicode (value)
       else:
         try:
@@ -141,14 +141,17 @@ class geasInstance:
       
     elif propertydef.dbType == "date":
       # Date property: Must be an mx.DateTime or at least parseable as such
-      if isinstance (value, types.UnicodeType):
+      if isinstance (value, unicode):
         value = mx.DateTime.ISO.ParseDateTime (value.encode ('utf-8') + \
                                                ' 00:00:00')
-      elif isinstance (value, types.StringType):
+      elif isinstance (value, str):
         value = mx.DateTime.ISO.ParseDateTime (value + ' 00:00:00')
 
-      elif sys.version_info [:2] >= (2, 3) and \
-          isinstance (value, datetime.date):
+      elif isinstance (value, datetime.datetime):
+        value = mx.DateTime.ISO.ParseDateTime ((value.date()).isoformat () \
+            + ' 00:00:00')
+
+      elif isinstance (value, datetime.date):
         value = mx.DateTime.ISO.ParseDateTime (value.isoformat () + ' 00:00:00')
 
       if isinstance (value, mx.DateTime.DateTimeType):
@@ -159,10 +162,10 @@ class geasInstance:
 
     elif propertydef.dbType == "time":
       # Time property: Must be an mx.DateTime or at least parseable as such
-      if isinstance (value, types.UnicodeType):
+      if isinstance (value, unicode):
         value = mx.DateTime.ISO.ParseDateTime ('0001-01-01 ' + \
                                                value.encode ('utf-8'))
-      elif isinstance (value, types.StringType):
+      elif isinstance (value, str):
         value = mx.DateTime.ISO.ParseDateTime ('0001-01-01 ' + value)
 
       elif sys.version_info [:2] >= (2, 3) and \
@@ -184,10 +187,10 @@ class geasInstance:
     elif propertydef.dbType == "datetime":
       # Date/Time property: Must be an mx.DateTime or at least parseable as
       # such
-      if isinstance (value, types.UnicodeType):
+      if isinstance (value, unicode):
         return mx.DateTime.ISO.ParseDateTime (value.encode ('utf-8'))
 
-      elif isinstance (value, types.StringType):
+      elif isinstance (value, str):
         return mx.DateTime.ISO.ParseDateTime (value)
 
       elif sys.version_info [:2] >= (2, 3) and \
@@ -221,7 +224,7 @@ class geasInstance:
   def __getValue (self, propertyname):
 
     #resolve indirect properties
-    elements = string.split (propertyname, '.')
+    elements = propertyname.split ('.')
     classdef = self.__classdef
     record   = self.__record
 
@@ -279,15 +282,15 @@ class geasInstance:
 
     # Do not call OnChange triggers while in OnInit code and when setting
     # time/user stamp fields
-    if regular and self.status () != 'initializing':
-      for proc in self.__classdef.procedures.values ():
-        if proc.gnue_name.upper () == 'ONCHANGE':
-          self.call (proc, {}, {'oldValue': self.__getValue (propertyname),
-                                'newValue': __value,
-                                'propertyName': propertydef.fullName})
+    if regular and self.state () != 'initializing':
+      for trigger in self.__classdef.OnChange:
+        self.call (trigger, {}, {'oldValue': self.__getValue (propertyname),
+                                 'newValue': __value,
+                                 'propertyName': propertydef.fullName})
 
     self.__record.putField (propertydef.column, __value)
 
+
   # ---------------------------------------------------------------------------
   # Set the values of a list of properties
   # ---------------------------------------------------------------------------
@@ -306,7 +309,7 @@ class geasInstance:
     # TODO: This should run in a separate process so that a segfaulting
     # procedure doesn't kill appserver.
     # (needs to be implemented as an option in gnue-common)
-    checktype (namespace, [types.NoneType, types.DictType])
+    checktype (namespace, [None, dict])
 
     # Create a session object which with the actual session id
     sess = Session.Session (self.__session.sm, id (self.__session),
@@ -345,7 +348,7 @@ class geasInstance:
       cx.bindFunction ('find',       sess.find)
       cx.bindFunction ('setcontext', sess.setcontext)
       cx.bindFunction ('new',        sess.new)
-      cx.bindFunction ('message',    sess.message)
+      cx.bindBuiltin ('message',    sess.message)
 
       # direct access to RPC API func.
       cx.bindFunction ('direct_request', self.__session.request)
@@ -355,7 +358,7 @@ class geasInstance:
       cx.bindFunction ('direct_call',    self.__session.call)
 
       # set context for the procedure
-      sess.setcontext (proceduredef.module.gnue_name)
+      sess.setcontext (proceduredef.gnue_module.gnue_name)
 
       method = cx.buildFunction (proceduredef.gnue_name, proceduredef.gnue_code,
                                  parameters)
@@ -384,22 +387,8 @@ class geasInstance:
     is raised.
     """
 
-    # First get all validation procedures of the class
-    ownValidate = None
-    triggers    = []
-    for proc in self.__classdef.procedures.values ():
-      if proc.gnue_name.upper () == 'ONVALIDATE':
-        if proc.module.fullName == self.__classdef.module.fullName:
-          ownValidate = proc
-        else:
-          triggers.append (proc)
-
-    # now call all triggers
-    for proc in triggers:
-      self.call (proc, None)
-
-    if ownValidate is not None:
-      self.call (ownValidate, None)
+    for trigger in self.__classdef.OnValidate:
+      self.call (trigger, None)
     
     # after finishing all OnValidate calls, have a look at the required fields
     for prop in self.__classdef.properties.values ():
@@ -443,40 +432,20 @@ class geasInstance:
   # Get the state of the instances record in the cache
   # ---------------------------------------------------------------------------
 
-  def status (self):
+  def state (self):
     """
     This function returns the current state of the instance.
 
     @return: state of the instance:
-
-    'initializing': new instance, OnInit still running
-
-    'initialized': new instance, OnInit finished, but no other modifications
-
-    'inserted': new instance, already modified
-
-    'changed': existing instance with modifications
-
-    'deleted': deleted instance
+      'initializing': new instance, OnInit still running
+      'initialized': new instance, OnInit finished, but no other modifications
+      'inserted': new instance, already modified
+      'changed': existing instance with modifications
+      'deleted': deleted instance
+      'clean': existing instance without any modifications
     """
 
-    return self.__record.status ()
-
-
-  # ---------------------------------------------------------------------------
-  # Touch the record in the cache
-  # ---------------------------------------------------------------------------
-
-  def touch (self, dirty = True):
-    """
-    This function touches the encapsulated record in the cache. If the record
-    has been initialized already it becomes 'commitable' this way.
-
-    @param dirty: if set to TRUE an 'initialized' record becomes 'commitable',
-        otherwise it might change to 'initializing' only.
-    """
-
-    self.__record.touch (dirty)
+    return self.__record.state ()
 
 
   # ---------------------------------------------------------------------------
@@ -504,7 +473,7 @@ class geasInstance:
         the value to be compared and 'descending' specifies the sort-direction.
     """
 
-    checktype (order, [types.NoneType, types.ListType])
+    checktype (order, [None, list])
     self.__order = order
 
 
