@@ -19,10 +19,9 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: geasList.py 7058 2005-02-23 16:54:24Z johannes $
+# $Id: geasList.py 7641 2005-06-22 13:58:32Z johannes $
 
 import geasInstance
-import string
 import time
 import copy
 
@@ -118,6 +117,8 @@ class geasList:
       if current is None:
         break
 
+    self.__length = len (self.__instances)
+
 
   # ---------------------------------------------------------------------------
   # Fill up instance list without any sorting/grouping
@@ -134,6 +135,8 @@ class geasList:
       if current is not None:
         self.__instances.append (current)
 
+    self.__length = len (self.__instances)
+
 
   # ---------------------------------------------------------------------------
   # wanna have more instances ?
@@ -278,12 +281,9 @@ class geasList:
       self.__recordset.close ()
       self.__recordset = None
 
-    self.__instances  = []
-    self.__unsorted   = []
-
-    if self.__condition is not None:
-      self.__condition.breakReferences ()
-      self.__condition = None
+    self.__instances = []
+    self.__unsorted  = []
+    self.__condition = None
 
 
   # ---------------------------------------------------------------------------
@@ -309,7 +309,7 @@ class geasList:
     # elements later, disturbing the following request.
     if len (caller._children):
       for c in caller._children:
-        new = GConditions.buildTreeFromList (c.prefixNotation ())
+        new = GConditions.buildConditionFromPrefix (c.prefixNotation ())
         new.parent = parent
         parent._children.append (new)
 
