@@ -19,19 +19,19 @@
 #
 # Copyright 2001-2005 Free Software Foundation
 #
-# $Id: geasRpcServer.py 7085 2005-03-03 11:08:01Z johannes $
+# $Id: geasRpcServer.py 7519 2005-05-04 12:59:09Z johannes $
 
 import time
 import os
 import sys
 import signal
-import gc
 
 import gnue.paths
 
 from gnue.common.apps import GConfig, i18n
+from gnue.common.apps.i18n import translate as _        # for epydoc
 from gnue.common.apps.GServerApp import GServerApp
-from gnue.common.datasources import GLoginHandler
+from gnue.common.datasources.GLoginHandler import SilentLoginHandler
 from gnue.common.rpc import server
 from gnue.common.utils.FileUtils import openResource
 
@@ -40,8 +40,6 @@ from gnue.appserver import geasSessionManager
 from gnue.appserver import geasConfiguration
 
 
-_GC_DEBUG = False  # Set this to True to activate garbage collection debugging
-
 # =============================================================================
 # RPC application class
 # =============================================================================
@@ -219,13 +217,8 @@ interface.""")],
     # Daemonize (if appropriate)
     GServerApp.run (self)
 
-    if _GC_DEBUG:
-      gc.enable ()
-      gc.set_debug (gc.DEBUG_LEAK)
-
     try:
       signal.signal (signal.SIGHUP, self._hangupServer)
-      signal.signal (signal.SIGUSR1, self._gcCheck)
       
     except AttributeError:
       pass
@@ -249,11 +242,12 @@ interface.""")],
     if hasattr (self, "sm"):
       return self.sm
 
-    loginhandler = geasLoginHandler ()
-    if self.OPTIONS ["username"]:
-      loginhandler.setLoginData (self.OPTIONS ["username"],
-                                 self.OPTIONS ["password"])
-    self.connections.setLoginHandler (loginhandler)
+    user  = self.OPTIONS ['username'] or 'gnue'
+    paswd = self.OPTIONS ['password'] or ''
+
+    loginHandler = SilentLoginHandler (_username = user, _password = paswd)
+    self.connections.setLoginHandler (loginHandler)
+
     self.sm = geasSessionManager.geasSessionManager (self.connections,
                                        gConfig ('modulepath'), True)
     return self.sm
@@ -271,51 +265,6 @@ interface.""")],
     if self.sm:
       self.sm.updateRepository (scanModules = True)
 
-  # ---------------------------------------------------------------------------
-  # Check the current state of the garbage collection
-  # ---------------------------------------------------------------------------
-
-  def _gcCheck (self, signal, frame):
-    """
-    This function starts a garbage collection, and lists all unreachable
-    objects not collected.
-    """
-
-    if not _GC_DEBUG:
-      print o(u_("Debugging garbage collection not activated."))
-      return
-
-    r = gc.collect ()
-
-    gf = open ('garbage.log', 'w')
-
-    print "=" * 70
-    print "GARBAGE:", r, ":"
-    gf.write ('GARBAGE: %d\n' % r)
-
-    n = 0
-    for x in gc.garbage:
-      n += 1
-      s = str(x)
-      if len(s) > 75: s = s[:75]
-      print type(x),"\n  ", s
-      gf.write ("%s: %s\n" % (type (x), repr (x)))
-
-      from gnue.common.datasources import GConditions
-      if isinstance (x, GConditions.GCondition):
-        gf.write ("Referrer of: %s\n" % x)
-        for y in gc.get_referrers (x):
-          gf.write ("--> %s: %s\n" % (type (y), y))
-
-        gf.write ("%s\n" % ("-" * 70))
-
-    gf.close ()
-    del gc.garbage [:]
-
-    print "-" * 70
-    print "GC dump done", n
-
-
   # ---------------------------------------------------------------------------
   # Self test
   # ---------------------------------------------------------------------------
@@ -361,19 +310,6 @@ These are the values of the first instance:
 
     print _('Selftest passed!')
 
-# =============================================================================
-# Login Handler
-# =============================================================================
-
-class geasLoginHandler (GLoginHandler.LoginHandler):
-  def setLoginData (self,username,password):
-    self._username = username
-    self._password = password
-
-  def getLogin (self, *arguments):
-    if hasattr (self, "_username"):
-      return {"_username": self._username, "_password": self._password}
-    return {"_username": "gnue", "_password": None}
 
 # =============================================================================
 # Main program
