@@ -19,11 +19,8 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: geasSession.py 7060 2005-02-23 18:48:56Z johannes $
+# $Id: geasSession.py 7641 2005-06-22 13:58:32Z johannes $
 
-from types import *
-
-import string
 import copy
 
 from gnue.common.apps import errors, i18n
@@ -31,7 +28,7 @@ from gnue.common.datasources import GConditions
 
 from gnue.appserver import data, geasList, geasInstance
 from gnue.appserver.geasAuthentication import AuthError
-from gnue.appserver.classrep.Property import PropertyNotFoundError
+from gnue.appserver.repository import PropertyNotFoundError
 
 
 # =============================================================================
@@ -55,11 +52,6 @@ class ListNotFoundError (errors.SystemError):
     msg = u_ ("Cannot find a list with ID '%s'") % listId
     errors.SystemError.__init__ (self, msg)
 
-class ConditionDataTypeError (errors.ApplicationError):
-  def __init__ (self, condition):
-    msg = u_("Invalid condition data type '%s'") % type (condition)
-    errors.ApplicationError.__init__ (self, msg)
-
 class NoFilterParamError (errors.UserError):
   def __init__ (self, filterName):
     msg = u_("No parameter for the filter %s specified") % filterName
@@ -83,7 +75,7 @@ class MultipleFilterValueError (errors.AdminError):
 class ValidationCyclesError (errors.ApplicationError):
   def __init__ (self, classlist):
     msg = u_("Maximum validation cycle reached. Classes in current cycle: %s")\
-          % string.join (classlist, ", ")
+          % ", ".join (classlist)
     errors.ApplicationError.__init__ (self, msg)
 
 # =============================================================================
@@ -115,15 +107,13 @@ class geasSession:
 
     self.sm     = sm                    # The session manager
     self.parameters = params
-    self.locale  = params.get ('language')
+    self.locale  = params.get ('_language')
     self.user    = None
     self.filters = {}
 
     self.nullFirstAsc = gConfig ('null_first_asc')
     self.nullFirstDsc = gConfig ('null_first_dsc')
 
-    gDebug (2, "Session-Params: %s" % params)
-
 
   # ---------------------------------------------------------------------------
   # Get a class definition from a class name and check access
@@ -135,7 +125,7 @@ class geasSession:
     if not self.__authAdapter.hasAccess (self, self.user, classname):
       raise AccessDeniedError, (classname)
 
-    return self.sm.classes [classname]
+    return self.sm.repository.classes [classname]
 
 
   # ---------------------------------------------------------------------------
@@ -154,7 +144,7 @@ class geasSession:
   def __getFieldname (self, classdef, propertyname, contentdict, add,
                       skipCalculated = True):
 
-    elements = string.split (propertyname, '.')
+    elements = propertyname.split ('.')
 
     a = u't0'                           # start with main table
     c = classdef
@@ -415,10 +405,10 @@ class geasSession:
 
     result = {'name': None, 'descending': False, 'ignorecase': False}
 
-    if isinstance (item, DictType):
+    if isinstance (item, dict):
       result = item
 
-    elif isinstance (item, TupleType) or isinstance (item, ListType):
+    elif isinstance (item, tuple) or isinstance (item, list):
       result ['name'] = item [0]
       if len (item) > 1: result ['descending'] = item [1]
       if len (item) > 2: result ['ignorecase'] = item [2]
@@ -479,7 +469,7 @@ class geasSession:
     # Only query the main table here.  geasInstance.get will do the rest.
     fields = {}
     for p in propertylist:
-      pdef = classdef.properties [string.split (p, '.') [0]]
+      pdef = classdef.properties [p.split ('.') [0]]
       if not fields.has_key (pdef.column) and not pdef.isCalculated:
         fields [pdef.column] = True
 
@@ -496,29 +486,22 @@ class geasSession:
 
   def __newInstance (self, classdef):
 
-    table    = classdef.table
-    record   = self.__connection.insertRecord (table)
+    table      = classdef.table
+    record     = self.__connection.insertRecord (table)
+    repository = self.sm.repository
 
     instance = geasInstance.geasInstance (self, self.__connection, record,
                                           classdef)
     if classdef.gnue_filter:
       fId   = classdef.gnue_filter.gnue_id
-      fName = self.sm.classes.find (fId).fullName
+      fName = classdef.gnue_filter.fullName
       instance.put ([fName], [self.__filterValue (fId)])
 
     instance.updateStamp (True)
 
-    onInits = {}
-    for proc in classdef.procedures.values ():
-      if proc.gnue_name.upper () == 'ONINIT':
-        onInits [proc.module.fullName] = proc
-
-    if onInits.has_key (classdef.module.fullName):
-      instance.call (onInits [classdef.module.fullName], None)
-      del onInits [classdef.module.fullName]
-
-    for proc in onInits.values ():
-      instance.call (proc, None)
+    # Fire all OnInit triggers of the class definition
+    for trigger in classdef.OnInit:
+      instance.call (trigger, None)
 
     # all modifications are 'serious' from now on 
     record.initialized ()
@@ -553,7 +536,7 @@ class geasSession:
             typelist.append ('procedure')
 
           else:
-            raise PropertyNotFoundError, (classdef.fullName, p)
+            raise PropertyNotFoundError, (p, classdef.fullName)
 
         result.append (typelist)
 
@@ -579,10 +562,6 @@ class geasSession:
       for object_id in obj_id_list:
         if object_id:
           instance = self.__findInstance (classdef, object_id, [])
-          # make sure an 'initialized' record get's commitable, even if no
-          # further access to the instance occurs
-          instance.touch ()
-
           new_object_id = object_id
 
         else:
@@ -594,7 +573,7 @@ class geasSession:
         i += 1
         result.append (new_object_id)
 
-        if instance.status () in ['changed', 'inserted']:
+        if instance.state () in ['changed', 'inserted']:
           instance.updateStamp ()
           self.__dirtyInstances [new_object_id] = instance
 
@@ -626,9 +605,9 @@ class geasSession:
 
       for objectId in obj_id_list:
         instance = self.__findInstance (classdef, objectId, [])
-        for proc in classdef.procedures.values ():
-          if proc.gnue_name.upper () == 'ONDELETE':
-            instance.call (proc, None)
+
+        for trigger in classdef.OnDelete:
+          instance.call (trigger, None)
 
       for object_id in obj_id_list:
         self.__connection.deleteRecord (classdef.table, object_id)
@@ -690,18 +669,11 @@ class geasSession:
     if condition is None:
       return (self.__filterCondition (classdef), None)
 
-    if isinstance (condition, ListType):
-      cTree = GConditions.buildTreeFromList (condition)
-    elif isinstance (condition, DictType):
-      cTree = GConditions.buildConditionFromDict (condition)
-    elif isinstance (condition, GConditions.GCondition):
-      cTree = condition
-    else:
-      raise ConditionDataTypeError, condition
+    cTree = GConditions.buildCondition (condition)
   
     if classdef.gnue_filter:
       useFilter  = True
-      filterName = self.sm.classes.find (classdef.gnue_filter.gnue_id).fullName
+      filterName = classdef.gnue_filter.fullName
       cList = cTree.findChildrenOfType ('GCCField', allowAllChildren = True)
       if len (cList):
         for item in cList:
@@ -729,8 +701,6 @@ class geasSession:
       else:
         asTrees.append (tree)
 
-    cTree.breakReferences ()
-
     # now create the final trees
     dbCond = None
     asCond = None
@@ -741,7 +711,7 @@ class geasSession:
 
       for tree in dbTrees:
         comb._children.append (tree)
-        tree._parent = comb
+        tree.setParent (comb)
 
     if len (asTrees):
       asCond = GConditions.GCondition ()
@@ -749,7 +719,7 @@ class geasSession:
 
       for tree in asTrees:
         comb._children.append (tree)
-        tree._parent = comb
+        tree.setParent (comb)
 
     return (dbCond, asCond)
 
@@ -762,7 +732,7 @@ class geasSession:
 
     if classdef.gnue_filter:
       filterId   = classdef.gnue_filter.gnue_id
-      filterName = self.sm.classes.find (filterId).fullName
+      filterName = classdef.gnue_filter.fullName
       filterCond = GConditions.buildConditionFromDict ( \
           {filterName: self.__filterValue (filterId)})
     else:
@@ -777,14 +747,14 @@ class geasSession:
 
   def __filterValue (self, filterId):
 
-    filterName = self.sm.classes.find (filterId).fullName
+    filterName = self.sm.repository.classes [filterId].fullName
     if not self.filters.has_key (filterName):
       filterValue = self.parameters.get (filterName, None)
       if filterValue is None:
         raise NoFilterParamError, (filterName)
 
-      cond = ['eq', ['field', 'gnue_id'], ['const', filterValue]]
-      fList = self.request (filterName, cond, [], ['gnue_id'])
+      cond   = {'gnue_id': filterValue}
+      fList  = self.request (filterName, cond, [], ['gnue_id'])
       idList = self.fetch (fList, 0, 5)
 
       if not len (idList):
