@@ -19,13 +19,10 @@
 #
 # Copyright 2001-2005 Free Software Foundation
 #
-# $Id: geasSessionManager.py 7104 2005-03-07 16:49:25Z johannes $
-
-from types import *
+# $Id: geasSessionManager.py 7641 2005-06-22 13:58:32Z johannes $
 
 import geasSession
 import geasAuthentication
-import classrep
 import geasFilter
 import os.path
 import fnmatch
@@ -34,6 +31,7 @@ import time
 from gnue import paths
 from gnue.common.datasources import GConnections
 from gnue.common.apps import i18n, GConfig, errors
+from gnue.appserver import repository
 from gnue.appserver.gcd import readgcd
 from gnue.appserver.gld import readgld
 
@@ -63,11 +61,7 @@ class geasSessionManager:
     self._modulepath  = modulepath or \
                         os.path.join (paths.data, "share", "gnue", "appserver")
 
-    #import profile
-    #prof = profile.Profile ()
-    #prof.runctx ('self.updateRepository (haltOnError = True)', globals (),
-        #locals ())
-    #prof.dump_stats ('appserver.profile')
+    self.repository = repository.Repository (self._internalSession)
     self.updateRepository (scanModules, haltOnError = True)
 
     cfg = gConfigDict (section = 'appserver')
@@ -127,22 +121,25 @@ class geasSessionManager:
     try:
       (gcd, gld) = self._scanModulePath ()
 
-      if len (gcd):
-        reader = readgcd.gcdReader (self._connections,
-                                    gConfig ('connection'), gcd)
-        reader.run ()
+      # Lookup dictionaries for reuse by readgld
+      lpModule    = {}
+      lpClass     = {}
+      lpProperty  = {}
+      lpProcedure = {}
 
-        modRS = reader.moduleRS
-        clsRS = reader.classRS
-        prpRS = reader.propRS
-        prcRS = reader.procRS
-      else:
-        (modRS, clsRS, prpRS, prcRS) = (None, None, None, None)
+      if len (gcd):
+        gcdreader = readgcd.gcdReader (self._connections,
+                                    gConfig ('connection'), gcd)
+        gcdreader.run ()
+
+        lpModule     = gcdreader.lpModule
+        lpClass      = gcdreader.lpClass
+        lpProperty   = gcdreader.lpProperty
+        lpProcedure  = gcdreader.lpProcedure
 
       if len (gld):
-        reader = readgld.gldReader (self._connections,
-                                    gConfig ('connection'), gld,
-                                    modRS, clsRS, prpRS, prcRS)
+        reader = readgld.gldReader (self._connections, gConfig ('connection'),
+            gld, lpModule, lpClass, lpProperty, lpProcedure)
         reader.run ()
 
     except:
@@ -170,7 +167,7 @@ class geasSessionManager:
 
     self._internalSession.rollback ()
 
-    classrep.init (self)
+    self.repository.load ()
 
     self._filter = geasFilter.geasFilter (self)
 
@@ -248,7 +245,8 @@ class geasSessionManager:
     sess = geasSession.geasSession (conn, self._authAdapter, self,
                                     authentication)
 
-    sess.login (authentication ['user'], authentication ['password'])
+    sess.login (authentication.get ('_username'),
+                authentication.get ('_password'))
 
     # We use the memory address of the geasSession object as session id.  This
     # is always unique and even thread safe.
