@@ -19,11 +19,11 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: classdef.py 6896 2005-01-13 15:51:21Z johannes $
+# $Id: classdef.py 7194 2005-03-13 10:41:15Z johannes $
 
 import mx
 
-from gnue.appserver.classrep import Namespace
+from gnue.appserver import repository
 from gnue.common.apps import errors
 
 # =============================================================================
@@ -57,7 +57,7 @@ class ClassDef:
     self.__session   = session
     self.__class     = klass
     self.__language  = language
-    self.classname   = Namespace.createName (klass.module.name, klass.name)
+    self.classname   = "%s_%s" % (klass.module.name, klass.name)
     self.classLabel  = "%s %s" % (klass.module.name.title (),
                                   klass.name.title ())
     self.isReference = asReference
@@ -96,6 +96,24 @@ class ClassDef:
     return len (search) and search [0][1] or None
 
 
+  # ---------------------------------------------------------------------------
+  # Release all resources and pointers held by a classdef instance
+  # ---------------------------------------------------------------------------
+
+  def release (self):
+    """
+    Release all references and pointers of a class definition instance. This
+    allows the garbage collection to work fine.
+    """
+
+    for item in self.properties + self.specials:
+      item.parent = None
+      if item.reference is not None:
+        item.reference.release ()
+
+      item.reference = None
+
+
   # ---------------------------------------------------------------------------
   # Create a sequence of languages to be processed for labels
   # ---------------------------------------------------------------------------
@@ -206,7 +224,7 @@ class ClassDef:
 
       for label in self.__session.find ('gnue_label', cond, [], pList):
 
-        full = Namespace.createName (label.gnue_property.module.name,
+        full = repository.createName (label.gnue_property.module.name,
                                      label.gnue_property.name)
         if full == 'gnue_id' and len (label.label):
           self.classLabel = label.label
@@ -278,7 +296,7 @@ class ClassDef:
     else:
       for item in self.properties:
         if '_' in item.propDef.gnue_type:
-          (refMod, refName) = Namespace.splitName (item.propDef.gnue_type)
+          (refMod, refName) = repository.splitName (item.propDef.gnue_type)
           refClass = self.__session.find ('gnue_class',
               {'gnue_name': refName, 'gnue_module.gnue_name': refMod}, [], [])
           cDef = ClassDef (self.__session, refClass [0], self.__language, True)
@@ -333,6 +351,7 @@ class ClassDef:
     return [(p, pageDict [p]) for p in pageNames]
 
 
+
     
 
 # =============================================================================
@@ -362,10 +381,10 @@ class Property:
     self.info     = None
     self.labelPos = None
 
-    self.fullName = Namespace.createName (propDef.gnue_module.gnue_name,
+    self.fullName = repository.createName (propDef.gnue_module.gnue_name,
                                           propDef.gnue_name)
     if isCalculated:
-      self.dbField = Namespace.createName (propDef.gnue_module.gnue_name,
+      self.dbField = repository.createName (propDef.gnue_module.gnue_name,
                                            propDef.gnue_name [3:])
     else:
       self.dbField = self.fullName
