@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: layout.py 6853 2005-01-04 13:37:27Z johannes $
+# $Id: layout.py 7463 2005-04-22 16:54:38Z reinhard $
 
 import math
 import operator
@@ -31,11 +31,6 @@ import sys
 
 class Manager:
 
-  _SPECIALS = {'gnue_createdate': {'Label': u_("Created"),
-                                   'Buddy': 'gnue_createuser'},
-               'gnue_modifydate': {'Label': u_("Last modified"),
-                                   'Buddy': 'gnue_modifyuser'}}
-
   # ---------------------------------------------------------------------------
   # Constructor
   # ---------------------------------------------------------------------------
@@ -47,6 +42,11 @@ class Manager:
     @param maxHeight: maximum height for a form
     """
 
+    self._SPECIALS = {'gnue_createdate': {'Label': u_("Created"),
+                                          'Buddy': 'gnue_createuser'},
+                      'gnue_modifydate': {'Label': u_("Last modified"),
+                                          'Buddy': 'gnue_modifyuser'}}
+
     self._maxWidth   = maxWidth
     self._maxHeight  = maxHeight
     self._classdef   = classdef
