@@ -19,13 +19,13 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: readgld.py 6998 2005-02-11 11:06:42Z johannes $
+# $Id: readgld.py 7829 2005-08-12 14:51:18Z reinhard $
 
 import sys
 import os
-import string
 
 from gnue.common.apps import GClientApp, i18n, errors
+from gnue.common.apps.i18n import translate as _        # for epydoc
 from gnue.common.utils.FileUtils import openResource
 from gnue.common.datasources import GDataSource, GConditions
 from gnue.common.utils.uuid import UUID
@@ -33,7 +33,7 @@ from gnue.common.utils.uuid import UUID
 from gnue.appserver import VERSION
 from gnue.appserver import geasConfiguration
 from gnue.appserver.gld import GLParser
-from gnue.appserver.classrep import Namespace
+from gnue.appserver import repository
 
 from gnue.common.apps import GBaseApp
 
@@ -50,10 +50,10 @@ class Error (errors.ApplicationError):
     text = []
 
     if files:
-      text.append (u_("In file(s) '%s':") % string.join (files, ', '))
+      text.append (u_("In file(s) '%s':") % ', '.join (files))
 
     text.append (message)
-    self.detail = string.join (text, os.linesep)
+    self.detail = os.linesep.join (text)
 
 class ClassNotFoundError (Error):
   def __init__ (self, classname, filename = None):
@@ -181,30 +181,33 @@ class gldReader:
   # Constructor
   # ---------------------------------------------------------------------------
 
-  def __init__ (self, connections, database, files, modRS = None, clsRS = None,
-      prpRS = None, prcRS = None):
+  def __init__ (self, connections, database, files, moduleLookup = None,
+      classLookup = None, propertyLookup = None, procedureLookup = None):
     """
-    Create a new instance of a gcd reader
+    Create a new instance of a gld reader
 
     @param connections: GConnections instance to be used
     @param database: name of the connection to use (in connection.conf)
     @param files: sequence of filenames to integerate
-    @param modRS: resultSet instance for 'gnue_modules'
-    @param clsRS: resultSet instance for 'gnue_class'
-    @param prpRS: resultSet instance for 'gnue_property'
-    @param prcRS: resultSet instance for 'gnue_procedure'
+    @param moduleLookup: lookup dictionary for 'gnue_modules'
+    @param classLookup: lookup dictionary for 'gnue_class'
+    @param propertyLookup: lookup dictionary for 'gnue_property'
+    @param procedureLookup: lookup dictionary for 'gnue_procedure'
     """
 
     self._connections = connections
     self._database    = database
     self._filenames   = files
-    self._files       = []
+    self._files       = {}
     self._uuidtype    = gConfig ('uuidtype', section = 'appserver').lower ()
 
-    self.__modRS      = modRS
-    self.__clsRS      = clsRS
-    self.__prpRS      = prpRS
-    self.__prcRS      = prcRS
+    self.lpModule    = moduleLookup
+    self.lpClass     = classLookup
+    self.lpProperty  = propertyLookup
+    self.lpProcedure = procedureLookup
+
+    self.labels   = {}
+    self.messages = {}
 
     if not len (self._filenames):
       raise StartupError, u_("No input file specified.")
@@ -214,10 +217,11 @@ class gldReader:
 
     try:
       for filename in files:
-        self._files.append (openResource (filename))
+        self._files [filename] = openResource (filename)
 
     except IOError, err:
-      for item in self._files:
+      while self._files:
+        (name, item) = self._files.popitem ()
         item.close ()
 
       raise StartupError, u_("Unable to open input file: %s") % \
@@ -234,25 +238,34 @@ class gldReader:
     updates the repository using these dictionaries.
     """
 
-    self.labels   = {}
-    self.messages = {}
+    self.labels.clear ()
+    self.messages.clear ()
 
     print _("Loading GNUe language definitions")
 
-    for ix in xrange (len (self._files)):
-      gDebug (1, "Loading gld file '%s'" % self._filenames [ix])
+    while self._files:
+      self.__currentFile, stream = self._files.popitem ()
 
       try:
-        self.__currentFile   = self._filenames [ix]
+        gDebug (1, "Loading gld file '%s'" % self.__currentFile)
+
         self.__currentModule = None
         self.__currentClass  = None
 
         # Load xml file and process all it's objects
-        schema = GLParser.loadFile (self._files [ix])
-        schema.walk (self.__iterateObjects)
+        schema = GLParser.loadFile (stream)
+        try:
+          schema.walk (self.__iterateObjects)
+
+        finally:
+          self.__currentModule = None
+          self.__currentClass  = None
+
+          schema = None
 
       finally:
-        self._files [ix].close ()
+        stream.close ()
+
 
     # Replace all references to modules, classes, properties and procedures
     self.__replaceReferences ()
@@ -262,6 +275,7 @@ class gldReader:
     self.__updateMessages ()
 
 
+
   # ---------------------------------------------------------------------------
   # Iterate over all top level elements
   # ---------------------------------------------------------------------------
@@ -297,8 +311,9 @@ class gldReader:
     """
 
     # This key makes sure to have no clashes
-    key = string.lower ("%s::%s.%s" % (self.__currentModule.language,
-                         self.__currentClass.fullName, item.fullName))
+    key = "%s::%s.%s" % (self.__currentModule.language,
+                         self.__currentClass.fullName, item.fullName)
+    key = key.lower ()
 
     if self.labels.has_key (key):
       raise DuplicateItemError, \
@@ -320,7 +335,7 @@ class gldReader:
 
 
   # ---------------------------------------------------------------------------
-  # Translate a message into a dictionary for later classrep update
+  # Translate a message into a dictionary for later class-repository update
   # ---------------------------------------------------------------------------
 
   def __translateMessage (self, item):
@@ -329,7 +344,7 @@ class gldReader:
     already listed, a DuplicateItemError will be raised.
     """
 
-    key = string.lower ("%s::%s") % (item.language, item.fullName)
+    key = "%s::%s" % (item.language.lower (), item.fullName.lower ())
     if self.messages.has_key (key):
       raise DuplicateItemError, \
           (self.__currentModule.language, item.fullName,
@@ -343,6 +358,7 @@ class gldReader:
         '_file'        : self.__currentFile}
 
 
+
   # ---------------------------------------------------------------------------
   # Get a dictionary with all keys listed in tags and values from sObject
   # ---------------------------------------------------------------------------
@@ -377,41 +393,49 @@ class gldReader:
     """
 
     # Build lookup dictionaries
-    modules    = self.__getModuleLookup ()
-    classes    = self.__getClassLookup (modules)
-    properties = self.__getPrLookup (self.__prpRS, 'gnue_property')
-    procedures = self.__getPrLookup (self.__prcRS, 'gnue_procedure')
+    if self.lpModule is None:
+      self.lpModule = self.__getModuleLookup ()
+
+    if self.lpClass is None:
+      self.lpClass = self.__getClassLookup ()
+
+    if self.lpProperty is None:
+      self.lpProperty = self.__getPropertyLookup ()
+
+    if self.lpProcedure is None:
+      self.lpProcedure = self.__getProcedureLookup ()
 
     # Replace references for all labels
     for label in self.labels.values ():
       classname = label ['gnue_class'].lower ()
-      classid   = classes.get (classname)
+      classid   = self.lpClass.get (classname)
       if classid is None:
         raise ClassNotFoundError, (label ['gnue_class'], label ['_file'])
 
       del label ['gnue_class']
 
       if label ['gnue_property'] is not None:
-        (module, name) = Namespace.splitName (label ['gnue_property'].lower ())
-        key  = "%s.%s_%s" % (classid, modules [module], name)
-        ckey = "%s.%s_get%s" % (classid, modules [module], name)
-        if properties.has_key (key):
-          label ['gnue_property'] = properties [key]
+        (module, name) = repository.splitName (label ['gnue_property'].lower ())
+        key  = "%s.%s_%s" % (classid, self.lpModule [module], name)
+        ckey = "%s.%s_get%s" % (classname, module, name)
+
+        if self.lpProperty.has_key (key):
+          label ['gnue_property']  = self.lpProperty [key]
           label ['gnue_procedure'] = None
 
-        elif procedures.has_key (ckey):
-          label ['gnue_procedure'] = procedures [ckey]
-          label ['gnue_property'] = None
+        elif self.lpProcedure.has_key (ckey):
+          label ['gnue_procedure'] = self.lpProcedure [ckey]
+          label ['gnue_property']  = None
 
         else:
           raise PropertyNotFoundError, \
               (classname, label ['gnue_property'], label ['_file'])
 
       elif label ['gnue_procedure'] is not None:
-        (module, name) = Namespace.splitName (label ['gnue_procedure'].lower ())
-        key = "%s.%s_%s" % (classid, modules [module], name)
-        ckey = "%s.%s_get%s" % (classid, modules [module], name)
-        ref = procedures.get (key) or procedures.get (ckey)
+        (module, name) = repository.splitName (label ['gnue_procedure'].lower())
+        key  = "%s.%s_%s" % (classname, module, name)
+        ckey = "%s.%s_get%s" % (classname, module, name)
+        ref  = self.lpProcedure.get (key, self.lpProcedure.get (ckey))
         if ref is None:
           raise ProcedureNotFoundError, \
               (classname, label ['gnue_procedure'], label ['_file'])
@@ -423,7 +447,7 @@ class gldReader:
 
     # Replace references for all messages
     for message in self.messages.values ():
-      message ['gnue_module'] = modules [message ['gnue_module'].lower ()]
+      message ['gnue_module'] = self.lpModule [message ['gnue_module'].lower ()]
       del message ['_file']
 
 
@@ -436,91 +460,132 @@ class gldReader:
     This function creates a lookup dictionary for modules. It's a mapping for
     id to name and vice versa.
 
-    @result: dictionary with id-to-name and name-to-id maps for modules
+    @return: dictionary with id-to-name and name-to-id maps for modules
     """
     
     result = {}
 
-    if self.__modRS is None:
-      src = self.__openSource ('gnue_module', ['gnue_id', 'gnue_name'])
-      self.__modRS = src.createResultSet ()
+    resultSet = self.__openSource ('gnue_module',
+                                   ['gnue_id', 'gnue_name']).createResultSet ()
+    try:
+      rec = resultSet.firstRecord ()
+      while rec is not None:
+        name = rec.getField ('gnue_name')
+        gid  = rec.getField ('gnue_id')
 
-    rec = self.__modRS.firstRecord ()
-    while rec is not None:
-      name = rec.getField ('gnue_name')
-      gid  = rec.getField ('gnue_id')
+        result [gid]           = name
+        result [name.lower ()] = gid
 
-      result [gid]           = name
-      result [name.lower ()] = gid
+        rec = resultSet.nextRecord ()
 
-      rec = self.__modRS.nextRecord ()
+      return result
 
-    return result
+    finally:
+      resultSet.close ()
 
 
   # ---------------------------------------------------------------------------
   # Create a lookup dictionary for classes
   # ---------------------------------------------------------------------------
 
-  def __getClassLookup (self, modules):
+  def __getClassLookup (self):
     """
     This function creates a lookup dictionary for classes, where the full
     qualified name is the key and the gnue_id is the value.
 
-    @param modules: lookup dictionary for modules
     @return: dictionary with class mapping
     """
 
     result = {}
 
-    if self.__clsRS is None:
-      src = self.__openSource ('gnue_class',
-                               ['gnue_id', 'gnue_module', 'gnue_name'])
-      self.__clsRS = src.createResultSet ()
+    resultSet = self.__openSource ('gnue_class',
+                    ['gnue_id', 'gnue_module', 'gnue_name']).createResultSet ()
+    try:
+      rec = resultSet.firstRecord ()
+      while rec is not None:
+        module = self.lpModule.get (rec.getField ('gnue_module'))
+        name   = rec.getField ('gnue_name')
+        gid    = rec.getField ('gnue_id')
+        key    = repository.createName (module, name)
 
-    rec = self.__clsRS.firstRecord ()
-    while rec is not None:
-      module = rec.getField ('gnue_module')
-      name   = rec.getField ('gnue_name')
-      key    = Namespace.createName (modules [module.lower ()], name)
-      result [key.lower ()] = rec.getField ('gnue_id')
+        result [key.lower ()] = gid
+        result [gid]          = key.lower ()
 
-      rec = self.__clsRS.nextRecord ()
+        rec = resultSet.nextRecord ()
 
-    return result
+      return result
+
+    finally:
+      resultSet.close ()
 
 
   # ---------------------------------------------------------------------------
-  # Create a lookup dictionary for properties or procedures
+  # Build a lookup-dictionary for properties
   # ---------------------------------------------------------------------------
 
-  def __getPrLookup (self, resultSet, table):
+  def __getPropertyLookup (self):
     """
-    This function creates a lookup dictionary for properties or procedures.
-
-    @param resultSet: a resultSet instance to use or None if a new resultSet
-        should be created. 
-    @param table: if no resultSet is given this tablename will be used to
-        create a new one.
-    @return: dictionary with the property- or procedure-mapping
+    This function creates a lookup dictionary for properties where the key is
+    constructed from "classid.moduleid_propertyname".
     """
 
     result = {}
 
-    if resultSet is None:
-      src = self.__openSource (table,
-                         ['gnue_id', 'gnue_module', 'gnue_class', 'gnue_name'])
-      resultSet = src.createResultSet ()
+    resultSet = self.__openSource ('gnue_property', ['gnue_id',
+          'gnue_module', 'gnue_class', 'gnue_name', 'gnue_type', 'gnue_length',
+          'gnue_scale', 'gnue_nullable', 'gnue_comment']).createResultSet ()
 
-    rec = resultSet.firstRecord ()
-    while rec is not None:
-      key = "%s.%s_%s" % (rec ['gnue_class'], rec ['gnue_module'],
-                          rec ['gnue_name'])
-      result [key.lower ()] = rec ['gnue_id']
+    try:
+      rec = resultSet.firstRecord ()
+      while rec is not None:
+        mid  = rec.getField ('gnue_module')
+        cid  = rec.getField ('gnue_class')
+        name = rec.getField ('gnue_name')
+        key  = "%s.%s_%s" % (cid, mid, name.lower ())
 
-      rec = resultSet.nextRecord ()
+        result [key] = rec.getField ('gnue_id')
 
-    return result
+        rec  = resultSet.nextRecord ()
+
+      return result
+
+    finally:
+      resultSet.close ()
+
+
+  # ---------------------------------------------------------------------------
+  # Build a lookup-dictionary for procedures
+  # ---------------------------------------------------------------------------
+
+  def __getProcedureLookup (self):
+    """
+    This function creates a procedure lookup dictionary where the key is built
+    from "classname.modulename_procedurename".
+    """
+
+    result = {}
+
+    resultSet = self.__openSource ('gnue_procedure', ['gnue_id',
+        'gnue_module', 'gnue_class', 'gnue_name', 'gnue_type', 'gnue_length',
+        'gnue_scale', 'gnue_nullable', 'gnue_comment', 'gnue_code',
+        'gnue_language']).createResultSet ()
+
+    try:
+      rec = resultSet.firstRecord ()
+      while rec is not None:
+        mName = self.lpModule.get (rec.getField ('gnue_module'))
+        cName = self.lpClass.get (rec.getField ('gnue_class'))
+        pName = rec.getField ('gnue_name')
+        key   = "%s.%s_%s" % (cName, mName, pName)
+
+        result [key.lower ()] = rec.getField ('gnue_id')
+
+        rec  = resultSet.nextRecord ()
+
+      return result
+
+    finally:
+      resultSet.close ()
 
 
   # ---------------------------------------------------------------------------
@@ -542,8 +607,7 @@ class gldReader:
                                           {'name'      : "dts_%s" % classname,
                                            'database'  : self._database,
                                            'table'     : classname,
-                                           'primarykey': 'gnue_id'},
-                                          unicodeMode = True)
+                                           'primarykey': 'gnue_id'})
 
 
   # ---------------------------------------------------------------------------
@@ -562,18 +626,22 @@ class gldReader:
     # Load and update all labels. To identify a label without gnue_id, use
     # 'gnue_language', 'gnue_property' and 'gnue_procedure'.
     rSet = src.createResultSet ()
-    cond = ['gnue_language', 'gnue_property', 'gnue_procedure']
-    stat = self.__processResultSet (rSet, self.labels, cond)
+    try:
+      cond = ['gnue_language', 'gnue_property', 'gnue_procedure']
+      stat = self.__processResultSet (rSet, self.labels, cond)
 
-    if stat [0] + stat [1]:
-      self._connections.commitAll ()
+      if stat [0] + stat [1]:
+        self._connections.commitAll ()
 
-    msg = u_("  Labels  : %(ins)3d inserted, %(upd)3d updated, %(kept)3d "
-             "unchanged.") \
-          % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]}
+      msg = u_("  Labels  : %(ins)3d inserted, %(upd)3d updated, %(kept)3d "
+               "unchanged.") \
+            % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]}
 
-    gDebug (1, msg)
-    print o(msg)
+      gDebug (1, msg)
+      print o(msg)
+
+    finally:
+      rSet.close ()
 
 
   # ---------------------------------------------------------------------------
@@ -591,18 +659,22 @@ class gldReader:
     # Load and update all messages. To identify a message without a gnue_id
     # 'gnue_language', 'gnue_module' and 'gnue_name' will be used.
     rSet = src.createResultSet ()
-    cond = ['gnue_language', 'gnue_module', 'gnue_name']
-    stat = self.__processResultSet (rSet, self.messages, cond)
+    try:
+      cond = ['gnue_language', 'gnue_module', 'gnue_name']
+      stat = self.__processResultSet (rSet, self.messages, cond)
 
-    if stat [0] + stat [1]:
-      self._connections.commitAll ()
+      if stat [0] + stat [1]:
+        self._connections.commitAll ()
 
-    msg = u_("  Messages: %(ins)3d inserted, %(upd)3d updated, %(kept)3d "
-                "unchanged.") \
-          % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]}
+      msg = u_("  Messages: %(ins)3d inserted, %(upd)3d updated, %(kept)3d "
+                  "unchanged.") \
+            % {'ins': stat [0], 'upd': stat [1], 'kept': stat [2]}
 
-    gDebug (1, msg)
-    print o(msg)
+      gDebug (1, msg)
+      print o(msg)
+
+    finally:
+      rSet.close ()
 
 
   # ---------------------------------------------------------------------------
@@ -653,6 +725,7 @@ class gldReader:
 
     if needPost:
       resultSet.post ()
+      resultSet.requery (False)
 
     return stat
 
@@ -712,6 +785,7 @@ class gldReader:
 
     if doPost and not deferPost:
       resultSet.post ()
+      resultSet.requery ()
 
     return doPost
 
