@@ -19,10 +19,10 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place 
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: App.py 6851 2005-01-03 20:59:28Z jcater $
+# $Id: App.py 7514 2005-05-03 12:26:35Z johannes $
 
 from gnue.common.rpc import client
-from gnue.common.apps import GClientApp
+from gnue.common.apps import GClientApp, i18n
 
 import Session
 
@@ -52,9 +52,18 @@ class App (GClientApp.GClientApp):
   # Create a new language interface session (LoginSession)
   # ---------------------------------------------------------------------------
 
-  def newSession (self, username, password):
+  def newSession (self, username, password, params = {}):
 
-    return Session.LoginSession (self._sm, username, password)
+    return Session.LoginSession (self._sm, username, password, params)
+
+
+  # ---------------------------------------------------------------------------
+  # Get the current set of filters
+  # ---------------------------------------------------------------------------
+
+  def getFilters (self, language = None):
+
+    return self._sm.getFilters (language or i18n.getuserlocale ())
 
 
   # ---------------------------------------------------------------------------
