@@ -19,10 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place 
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: Session.py 7104 2005-03-07 16:49:25Z johannes $
-
-import types
-import string
+# $Id: Session.py 7641 2005-06-22 13:58:32Z johannes $
 
 from ObjectList import ObjectList
 from Object import Object
@@ -69,6 +66,8 @@ class Session:
     self.parameters   = params
     self.classdefs    = {}
 
+    gDebug (6, "Built session %s with params %s" % (sessionId, params))
+
 
   # -------------------------------------------------------------------------
   # Get the session's sessionManager
@@ -176,7 +175,7 @@ class Session:
     @return: fully qualified version of name
     """
 
-    return string.join ([self.qualify (p) for p in name.split ('.')], ".")
+    return ".".join ([self.qualify (p) for p in name.split ('.')])
 
 
   # -------------------------------------------------------------------------
@@ -365,12 +364,12 @@ class Session:
     if not current in languages:
       languages.insert (0, current)
 
-    (module, message) = self.qualify (messageName).lower ().split ('_')
+    (module, message) = self.qualify (messageName).upper ().split ('_')
 
     for lang in languages:
-      cond = ['and', ['eq', ['lower', ['field', 'gnue_module.gnue_name']],
+      cond = ['and', ['eq', ['upper', ['field', 'gnue_module.gnue_name']],
                             ['const', module]],
-                     ['eq', ['lower', ['field', 'gnue_name']],
+                     ['eq', ['upper', ['field', 'gnue_name']],
                             ['const', message]],
                      ['eq', ['field', 'gnue_language'], ['const', lang]]]
 
@@ -379,7 +378,7 @@ class Session:
         text = result [0].gnue_text
 
         if len (args):
-          if isinstance (args [0], types.DictType):
+          if isinstance (args [0], dict):
             text = text % args [0]
           else:
             text = text % args
@@ -393,25 +392,34 @@ class Session:
 # =============================================================================
 # Login-Session creates a new Session-Id using 'user' and 'password'
 # =============================================================================
+
 class LoginSession (Session):
 
   # -------------------------------------------------------------------------
   # Constructor
   # -------------------------------------------------------------------------
+
   def __init__ (self, sessionManager, user, password, params = {}):
-    Session.__init__ (self, sessionManager,
-                  sessionManager.open ({'user': user, 'password': password}),
-                  params)
+
+    parameters = {'_username': user, '_password': password}
+    parameters.update (params)
+
+    Session.__init__ (self, sessionManager, sessionManager.open (parameters),
+        params)
+
 
 
 # =============================================================================
 # InternalSession uses 0 as Session-Id
 # =============================================================================
-class InternalSession(Session):
+
+class InternalSession (Session):
 
   # -------------------------------------------------------------------------
   # Constructor
   # -------------------------------------------------------------------------
+
   def __init__ (self, sessionManager, params = {}):
+
     Session.__init__ (self, sessionManager, 0, params)
 
