@@ -19,12 +19,13 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: test.py 6851 2005-01-03 20:59:28Z jcater $
+# $Id: test.py 7519 2005-05-04 12:59:09Z johannes $
 
 import os
 from gnue.common.apps.GClientApp import GClientApp
-from gnue.common.datasources.GLoginHandler import LoginHandler
+from gnue.common.datasources.GLoginHandler import SilentLoginHandler
 from gnue.appserver.geasSessionManager import geasSessionManager
+from gnue.appserver import geasConfiguration
 
 # =============================================================================
 # Test application
@@ -36,7 +37,7 @@ class testApp (GClientApp):
   # Constructor
   # ---------------------------------------------------------------------------
   def __init__ (self, connections = None):
-    ConfigOptions = {}
+    ConfigOptions = geasConfiguration.ConfigOptions
     GClientApp.__init__ (self, connections, 'appserver', ConfigOptions)
 
 
@@ -44,7 +45,8 @@ class testApp (GClientApp):
   # Create a session manager object
   # ---------------------------------------------------------------------------
   def getSessionManager (self):
-    self.connections.setLoginHandler (testLoginHandler ())
+    self.connections.setLoginHandler (SilentLoginHandler (_username = 'gnue',
+                                                          _password = ''))
 
     print "Creating session manager ..."
     sm = geasSessionManager (self.connections)
@@ -118,13 +120,6 @@ class testApp (GClientApp):
     print "Ok"
 
 
-# =============================================================================
-# Login Handler
-# =============================================================================
-class testLoginHandler (LoginHandler):
-  def getLogin (self, *arguments):
-    return {"_username": "gnue", "_password": None}
-
 
 # =============================================================================
 # Let it rock
